from __future__ import annotations

import json
import logging
import os
import pathlib
import traceback
from datetime import datetime
from importlib.metadata import version
from time import time
from typing import List, Union

import datasets

from ..abstasks import *
from ..abstasks import AbsTask, LangMapping
from ..tasks import *

logger = logging.getLogger(__name__)


class MTEB:
    def __init__(
        self,
        task_types: List[str] | None = None,
        task_categories: List[str] | None = None,
        task_langs: List[str] | None = None,
        tasks: List[Union[str, AbsTask]] | None = None,
        version=None,
        err_logs_path="error_logs.txt",
        **kwargs,
    ):
        """Create an Evaluation pipeline. The tasks selected
        depends on the parameters. One can specify the tasks types
        they want to evaluate (e.g. Clustering, Retrieval, etc.)
        the categories of tasks they want (e.g. Sentence2Sentence,
        Sentence2Paragraph, etc.) and the version of the benchmark.
        The selected tasks will be the tasks satisfying conditions
        from the 3 arguments. Alternatively, one can specify a list
        of tasks to be evaluated with the `tasks` argument. If
        `tasks` is specified, we filter tasks based on `task_langs`

        Args:
            task_types: List of task types (Clustering, Retrieval..) to be evaluated. If None, all tasks will be evaluated
            task_categories: List of task categories (s2s, p2p..) to be evaluated. If None, all tasks will be evaluated
            task_langs: List of languages to be evaluated. if None, all languages will be evaluated. ["eng-Latn", "deu_Latn"] will evaluate on all tasks
                with these languages.
            tasks: List of tasks to be evaluated. If specified, we filter tasks based on `task_langs` only
            version: Version of the benchmark to use. If None, latest is used
            err_logs_path: Path to save error logs
            kwargs: Additional arguments to be passed to the tasks
        """
        if tasks is not None:
            self._tasks = tasks
            assert (
                task_types is None and task_categories is None
            ), "Cannot specify both `tasks` and `task_types`/`task_categories`"
        else:
            self._task_types = task_types
            self._task_categories = task_categories
            self._tasks = None

        self._task_langs = task_langs if task_langs is not None else []
        if isinstance(self._task_langs, str):
            self._task_langs = [self._task_langs]

        self._extend_lang_code()
        self._extend_lang_pairs()  # add all possible pairs

        self._version = version
        self.err_logs_path = err_logs_path

        self.select_tasks(**kwargs)

    @property
    def available_tasks(self):
        return [x.metadata_dict["name"] for x in self.tasks_cls]

    @property
    def available_task_types(self):
        return set([x.metadata_dict["type"] for x in self.tasks_cls])

    @property
    def available_task_categories(self):
        return set([x.metadata_dict["category"] for x in self.tasks_cls])

    def _extend_lang_code(self):
        # add all possible language codes
        for lang in set(self._task_langs):
            if lang in LangMapping.LANG_MAPPING:
                self._task_langs += LangMapping.LANG_MAPPING[lang]

    def _extend_lang_pairs(self):
        # add all possible language pairs
        langs = set(self._task_langs)
        for x in langs:
            if "-" not in x:
                for y in langs:
                    if "-" not in y:
                        pair = f"{x}-{y}"
                        if pair not in langs:
                            self._task_langs.append(pair)
        return

    def _display_tasks(self, task_list, name=None):
        from rich.console import Console

        # disable logging for other ranks
        if int(os.getenv("RANK", 0)) != 0:
            return

        console = Console()
        if name:
            console.rule(f"[bold]{name}\n", style="grey15")
        for task_type in self.available_task_types:
            current_type_tasks = list(
                filter(lambda x: x.metadata_dict["type"] == task_type, task_list)
            )
            if len(current_type_tasks) == 0:
                continue
            else:
                console.print(f"[bold]{task_type}[/]")
                for task in current_type_tasks:
                    prefix = "    - "
                    name = f"{task.metadata_dict['name']}"
                    category = f", [italic grey39]{task.metadata_dict['category']}[/]"
                    multilingual = (
                        f", [italic red]multilingual {len(task.langs)} / {len(task.metadata_dict['eval_langs'])} langs[/]"
                        if task.is_multilingual
                        else ""
                    )
                    crosslingual = (
                        f", [italic cyan]crosslingual {len(task.langs)} / {len(task.metadata_dict['eval_langs'])} pairs[/]"
                        if task.is_crosslingual
                        else ""
                    )
                    console.print(
                        f"{prefix}{name}{category}{multilingual}{crosslingual}"
                    )
                console.print("\n")

    @classmethod
    def mteb_tasks(cls):
        """Get all tasks available in the MTEB."""
        instance = cls()
        instance._display_tasks(instance.tasks_cls, name="MTEB tasks")

    def print_selected_tasks(self):
        """Print the selected tasks."""
        self._display_tasks(self.tasks, name="Selected tasks")

    def select_tasks(self, **kwargs):
        """Select the tasks to be evaluated."""
        # Get all existing tasks
        tasks_categories_cls = [cls for cls in AbsTask.__subclasses__()]
        self.tasks_cls = [
            cls(langs=self._task_langs, **kwargs)
            for cat_cls in tasks_categories_cls
            for cls in cat_cls.__subclasses__()
            if cat_cls.__name__.startswith("AbsTask")
        ]

        # If `task_list` is specified, select list of tasks
        if self._tasks is not None:
            self.tasks = list(
                filter(
                    lambda x: (x.metadata_dict["name"] in self._tasks), self.tasks_cls
                )
            )
            if len(self.tasks) != len(self._tasks):
                tasks_known = set([x.metadata_dict["name"] for x in self.tasks_cls])
                tasks_unknown = (
                    set(x for x in self._tasks if isinstance(x, str)) - tasks_known
                )
                if tasks_unknown:
                    unknown_str, known_str = (
                        ",".join(sorted(list(tasks_unknown))),
                        ",".join(sorted(list(tasks_known))),
                    )
                    logger.warning(
                        f"WARNING: Unknown tasks: {unknown_str}. Known tasks: {known_str}."
                    )
            # add task if subclass of mteb.tasks
            self.tasks.extend([x for x in self._tasks if isinstance(x, AbsTask)])
            return

        # Otherwise use filters to select tasks
        filtered_tasks = filter(
            lambda x: (self._task_types is None)
            or (x.metadata_dict["type"] in self._task_types),
            self.tasks_cls,
        )
        filtered_tasks = filter(
            lambda x: (self._task_categories is None)
            or (x.metadata_dict["category"] in self._task_categories),
            filtered_tasks,
        )
        filtered_tasks = filter(
            lambda x: (self._version is None)
            or (x.metadata_dict["version"] >= self._version),
            filtered_tasks,
        )
        # keep only tasks with at least one language in the filter
        filtered_tasks = filter(
            lambda x: (not (self._task_langs))
            or (len(set(x.metadata_dict["eval_langs"]) & set(self._task_langs)) > 0),
            filtered_tasks,
        )

        # Get final list of tasks
        self.tasks = list(filtered_tasks)

    def load_tasks_data(self):
        """Load datasets for the selected tasks."""
        logger.info(f"\n\n## Loading datasets for {len(self.tasks)} tasks")
        for task in self.tasks:
            logger.info(f"\n# Loading dataset for {task.metadata_dict['name']}")
            task.load_data()

    def run(
        self,
        model,
        verbosity=1,
        output_folder="results/result",
        eval_splits=None,
        overwrite_results=False,
        raise_error: bool = True,
        **kwargs,
    ):
        """Run the evaluation pipeline on the selected tasks.

        Parameters
        ----------
        model:
            Model to be used for evaluation
        verbosity: int
            Verbosity level. Default is 1.
            0: print tasks tqdm progress bar
            1: print tasks tqdm progress bar and scores
            2: print everything (including datasets loading)
        output_folder: str
            Folder where the results will be saved
        raise_error: bool
            Whether to raise an error if an exception occurs during evaluation.
        :return: Returns a dictionary of task names and corresponding metrics results.
        """
        # Set logging
        if verbosity < 2:
            datasets.logging.set_verbosity(40)
            datasets.logging.disable_progress_bar()

        # Create output folder
        if output_folder is not None:
            pathlib.Path(output_folder).mkdir(parents=True, exist_ok=True)

        # Run selected tasks
        logger.info(f"\n\n## Evaluating {len(self.tasks)} tasks:")
        self.print_selected_tasks()
        evaluation_results = {}
        original_tasks = (
            self.tasks.copy()
        )  # save them in case we re-use the object (e.g. for reranking)
        while len(self.tasks) > 0:
            task = self.tasks[0]
            logger.info(
                f"\n\n********************** Evaluating {task.metadata_dict['name']} **********************"
            )

            # skip evaluation if results folder exists and overwrite_results is False
            if output_folder is not None:
                save_path = os.path.join(
                    output_folder,
                    f"{task.metadata_dict['name']}{task.save_suffix}.json",
                )
                if os.path.exists(save_path) and overwrite_results is False:
                    logger.warning(
                        f"WARNING: {task.metadata_dict['name']} results already exists. Skipping."
                    )
                    del self.tasks[0]
                    continue

            try:
                task_eval_splits = (
                    eval_splits
                    if eval_splits is not None
                    else task.metadata_dict.get("eval_splits", [])
                )

                # load data
                logger.info(f"Loading dataset for {task.metadata_dict['name']}")
                task.check_if_dataset_is_superseeded()
                task.load_data(eval_splits=task_eval_splits, **kwargs)

                # run evaluation
                task_results = {
                    "mteb_version": version("mteb"),  # noqa: F405
                    "dataset_revision": task.metadata_dict["dataset"].get(
                        "revision", None
                    ),
                    "mteb_dataset_name": task.metadata_dict["name"],
                }
                for split in task_eval_splits:
                    tick = time()
                    results = task.evaluate(
                        model, split, output_folder=output_folder, **kwargs
                    )
                    tock = time()
                    logger.info(
                        f"Evaluation for {task.metadata_dict['name']} on {split} took {tock - tick:.2f} seconds"
                    )
                    results["evaluation_time"] = round(tock - tick, 2)
                    task_results[split] = results
                    if verbosity >= 1:
                        logger.info(f"Scores: {results}")

                # save results
                if output_folder is not None:
                    with open(save_path, "w") as f_out:
                        json.dump(task_results, f_out, indent=2, sort_keys=True)

                evaluation_results[task.metadata_dict["name"]] = task_results

            except Exception as e:
                logger.error(
                    f"Error while evaluating {task.metadata_dict['name']}: {e}"
                )
                if raise_error:
                    raise e
                logger.error(
                    f"Please check all the error logs at: {self.err_logs_path}"
                )
                with open(self.err_logs_path, "a") as f_out:
                    f_out.write(f"{datetime.now()} >>> {task.metadata_dict['name']}\n")
                    f_out.write(traceback.format_exc())
                    f_out.write("\n\n")

            # empty memory
            del self.tasks[0]

        # restore original tasks
        self.tasks = original_tasks
        return evaluation_results



from __future__ import annotations

import logging
import random
from typing import Any, Dict

import numpy as np
import sklearn
import sklearn.cluster
from datasets import DatasetDict
from sklearn.metrics.cluster import v_measure_score

from .AbsTask import AbsTask

logger = logging.getLogger(__name__)

Split = str
HFLang = str
MultilingualDataset = Dict[HFLang, DatasetDict]
Scores = Dict[str, Any]


def evaluate_clustering_bootstrapped(
    embeddings: np.ndarray,
    labels: list[str],
    n_clusters: int,
    cluster_size: int,
    kmean_batch_size: int,
    rng_state: random.Random = random.Random(),
) -> list[float]:
    """Bootstrapped evaluation of clustering performance using V-measure.

    The bootstrapping is done by sampling N samples from the corpus and clustering them. It is done without replacement to get a diverse set of
    samples.
    """
    n_embeddings = embeddings.shape[0]
    labels_arr = np.array(labels)

    v_measures = []

    clustering_model = sklearn.cluster.MiniBatchKMeans(
        n_clusters=len(set(labels)),
        batch_size=kmean_batch_size,
        n_init="auto",
    )

    for _ in range(n_clusters):
        # sample N samples from the corpus with replacement
        cluster_indices = rng_state.choices(range(n_embeddings), k=cluster_size)

        _embeddings = embeddings[cluster_indices]
        _labels = labels_arr[cluster_indices]
        clustering_model.fit(_embeddings)
        cluster_assignment = clustering_model.labels_
        v_measure = v_measure_score(_labels, cluster_assignment)
        v_measures.append(v_measure)

    return v_measures


class AbsTaskClusteringFast(AbsTask):
    """Abstract class for Clustering tasks.

    This class embeds the corpus sentences then samples N samples from the corpus and clusters them.
    The similarity then is calculated using the V-measure metric, which is invariant to the permutation of the labels.
    This approach is then repeated K times.

    self.load_data() must generate a huggingface dataset with a split matching self.metadata_dict["eval_splits"], and assign it to self.dataset.
    It must contain the following columns:
        sentences: list[str]
        labels: list[str]
    """

    max_documents_to_embed = 16_384
    max_documents_per_cluster = 2048
    n_clusters = 10
    k_mean_batch_size = 512

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def _add_main_score(self, scores):
        if self.metadata_dict["main_score"] in scores:
            scores["main_score"] = scores[self.metadata_dict["main_score"]]
        else:
            logger.warn(
                f"main score {self.metadata_dict['main_score']} not found in scores {scores.keys()}"
            )

    def evaluate(self, model, split="test", **kwargs) -> Scores | Dict[HFLang, Scores]:
        lang: HFLang
        multilingual_ds: MultilingualDataset
        self.dataset: MultilingualDataset | DatasetDict
        ds: DatasetDict

        if not self.data_loaded:
            self.load_data()

        if self.is_multilingual:
            multilingual_ds = self.dataset  # type: ignore

            multilingual_scores: dict[HFLang, Scores] = {}
            for lang in self.dataset:  # type: ignore
                logger.info(
                    f"\nTask: {self.metadata.name}, split: {split}, language: {lang}. Running..."
                )
                _ds = multilingual_ds[lang]
                multilingual_scores[lang] = self._evaluate_monolingual(
                    model, multilingual_ds[lang], split, **kwargs
                )
                return multilingual_scores
        logger.info(f"\nTask: {self.metadata.name}, split: {split}. Running...")

        ds = self.dataset  # type: ignore
        scores = self._evaluate_monolingual(model, ds, split, **kwargs)
        return scores

    def _evaluate_monolingual(
        self, model, dataset: DatasetDict, split: Split = "test", **kwargs: Any
    ) -> dict[str, float | list[float]]:
        _dataset = dataset[split]

        rng_state = random.Random(self.seed)
        example_indices = rng_state.sample(
            range(len(_dataset)), k=self.max_documents_to_embed
        )
        downsampled_dataset = _dataset.select(example_indices)

        logger.info(f"Encoding {len(downsampled_dataset)} sentences...")

        embeddings = model.encode(downsampled_dataset["sentences"])

        v_measures = evaluate_clustering_bootstrapped(
            embeddings,
            downsampled_dataset["labels"],
            n_clusters=self.n_clusters,
            cluster_size=self.max_documents_per_cluster,
            kmean_batch_size=self.k_mean_batch_size,
            rng_state=rng_state,
        )

        return {"v_measures": v_measures, "v_measure": float(np.mean(v_measures))}



from __future__ import annotations

from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class NordicLangClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="NordicLangClassification",
        description="A dataset for Nordic language identification.",
        reference="https://aclanthology.org/2021.vardial-1.8/",
        dataset={
            "path": "strombergnlp/nordic_langid",
            "revision": "e254179d18ab0165fdb6dbef91178266222bee2a",
            "name": "10k",
        },
        type="Classification",
        category="s2s",
        eval_splits=["test"],
        eval_langs=[
            "nob-Latn",
            "nno-Latn",
            "dan-Latn",
            "swe-Latn",
            "isl-Latn",
            "fao-Latn",
        ],
        main_score="accuracy",
        date=None,
        form=None,
        domains=None,
        task_subtypes=None,
        license=None,
        socioeconomic_status=None,
        annotations_creators=None,
        dialect=None,
        text_creation=None,
        bibtex_citation=None,
        n_samples={"test": 3000},
        avg_character_length={"test": 78.2},
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["n_experiments"] = 10
        metadata_dict["samples_per_label"] = 32
        return metadata_dict

    def dataset_transform(self):
        self.dataset = self.dataset.rename_column("sentence", "text")
        self.dataset = self.dataset.rename_column("language", "label")



from __future__ import annotations

from typing import Any

import datasets

from mteb.abstasks import AbsTaskBitextMining, CrosslingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata

# Mapping from ISO 639-3 + script to ISO 639-1 used in NTREX
_ISO6393_to_ISO6391 = {
    "ara_Arab": "ar",
    "deu_Latn": "de",
    "eng_Latn": "en",
    "spa_Latn": "es",
    "fra_Latn": "fr",
    "hin_Deva": "hi",
    "ind_Latn": "id",
    "ita_Latn": "it",
    "jpn_Jpan": "ja",
    "kor_Hang": "ko",
    "por_Latn": "pt",
    "rus_Cyrl": "ru",
    "tha_Thai": "th",
    "tur_Latn": "tr",
    "vie_Latn": "vi",
    "zho_Hans": "zh",
}

_SPLIT = ["train"]

# number of sentences to use for evaluation
_N = 1997


def extend_lang_pairs() -> dict[str, list[str]]:
    # add all possible language pairs
    hf_lang_subset2isolang = {}
    for x in _ISO6393_to_ISO6391.keys():
        if "-" not in x:
            for y in _ISO6393_to_ISO6391.keys():
                if x != y:
                    pair = f"{x}-{y}"
                    hf_lang_subset2isolang[pair] = [
                        x.replace("_", "-"),
                        y.replace("_", "-"),
                    ]
    return hf_lang_subset2isolang


_EVAL_LANGS = extend_lang_pairs()


class NTREXBitextMining(AbsTaskBitextMining, CrosslingualTask):
    metadata = TaskMetadata(
        name="NTREXBitextMining",
        dataset={
            "path": "xianf/NTREX",
            "revision": "6047adbf00ceeab9b543bd0bd779b99866deddc0",
            "trust_remote_code": True,
        },
        description="NTREX is a News Test References for MT Evaluation from English into a total of 128 target languages.",
        reference="https://huggingface.co/datasets/xianf/NTREX",
        type="BitextMining",
        category="s2s",
        eval_splits=_SPLIT,
        eval_langs=_EVAL_LANGS,
        main_score="f1",
        date=("2022-11-01", "2022-11-01"),
        form=["written"],
        domains=["News"],
        task_subtypes=[],
        license="CC-BY-SA-4.0",
        socioeconomic_status="medium",
        annotations_creators="expert-annotated",
        dialect=[],
        text_creation="created",
        n_samples={"train": _N},
        avg_character_length={"train": 120},
        bibtex_citation="""
@inproceedings{federmann-etal-2022-ntrex,
    title = "{NTREX}-128 {--} News Test References for {MT} Evaluation of 128 Languages",
    author = "Federmann, Christian and Kocmi, Tom and Xin, Ying",
    booktitle = "Proceedings of the First Workshop on Scaling Up Multilingual Evaluation",
    month = "nov",
    year = "2022",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2022.sumeval-1.4",
    pages = "21--24",
}
""",
    )

    def load_data(self, **kwargs: Any) -> None:
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.dataset = {}

        all_data = {
            l: datasets.load_dataset(
                name=_ISO6393_to_ISO6391[l],
                split=_SPLIT,
                **self.metadata_dict["dataset"],
            )[0]
            for l in _ISO6393_to_ISO6391.keys()
        }

        for lang in self.langs:
            l1, l2 = lang.split("-")
            l1_data = all_data[l1].rename_column("text", "sentence1")
            l2_data = all_data[l2].rename_column("text", "sentence2")
            assert l1_data.num_rows == l2_data.num_rows
            # Combine languages
            data = l1_data.add_column("sentence2", l2_data["sentence2"])
            self.dataset[lang] = datasets.DatasetDict({"train": data})

        self.data_loaded = True



from __future__ import annotations

from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata

_LANGS = [
    "eng-Latn",
    "pol-Latn",
    "ron-Latn",
    "glg-Latn",
    "lit-Latn",
    "bre-Latn",
    "mlt-Latn",
    "spa-Latn",
    "swa-Latn",
    "slv-Latn",
    "msa-Latn",
    "tgl-Latn",
    "ita-Latn",
    "est-Latn",
    "mlg-Latn",
    "xho-Latn",
    "que-Latn",
    "eus-Latn",
    "nor-Latn",
    "cym-Latn",
    "cat-Latn",
    "fra-Latn",
    "nno-Latn",
    "nld-Latn",
    "ido-Latn",
    "por-Latn",
    "fin-Latn",
    "deu-Latn",
    "dan-Latn",
]


class ToxicChatClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="ToxicChatClassification",
        description="""This dataset contains toxicity annotations on 10K user
            prompts collected from the Vicuna online demo. We utilize a human-AI
            collaborative annotation framework to guarantee the quality of annotation
            while maintaining a feasible annotation workload. The details of data
            collection, pre-processing, and annotation can be found in our paper.
            We believe that ToxicChat can be a valuable resource to drive further
            advancements toward building a safe and healthy environment for user-AI
            interactions.
            Only human annotated samples are selected here.""",
        reference="https://aclanthology.org/2023.findings-emnlp.311/",
        dataset={
            "path": "lmsys/toxic-chat",
            "name": "toxicchat0124",
            "revision": "3e0319203c7162b9c9f8015b594441f979c199bc",
        },
        type="Classification",
        category="s2s",
        eval_splits=["test"],
        eval_langs=_LANGS,
        main_score="accuracy",
        date=("2023-10-26", "2024-01-31"),
        form=["written"],
        domains=["Constructed"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-4.0",
        socioeconomic_status="high",
        annotations_creators="expert-annotated",
        dialect=[],
        text_creation="found",
        bibtex_citation="""@misc{lin2023toxicchat,
            title={ToxicChat: Unveiling Hidden Challenges of Toxicity Detection in Real-World User-AI Conversation},
            author={Zi Lin and Zihan Wang and Yongqi Tong and Yangkun Wang and Yuxin Guo and Yujia Wang and Jingbo Shang},
            year={2023},
            eprint={2310.17389},
            archivePrefix={arXiv},
            primaryClass={cs.CL}
        }""",
        n_samples={"test": 1427},
        avg_character_length={"test": 189.4},
    )

    def dataset_transform(self):
        # only use human-annotated data
        self.dataset = self.dataset["test"].filter(lambda x: x["human_annotation"])
        keep_cols = ["user_input", "toxicity"]
        rename_dict = dict(zip(keep_cols, ["text", "label"]))
        remove_cols = [col for col in self.dataset.column_names if col not in keep_cols]
        self.dataset = self.dataset.rename_columns(rename_dict)
        self.dataset = self.dataset.remove_columns(remove_cols)
        self.dataset = self.dataset.class_encode_column("label")
        self.dataset = self.dataset.train_test_split(
            test_size=0.5, seed=self.seed, stratify_by_column="label"
        )  # balanced sampling across types of hate speech



from __future__ import annotations

import asyncio
import logging
from unittest.mock import Mock, patch

import aiohttp
import pytest

from mteb import MTEB
from mteb.abstasks import AbsTask
from mteb.abstasks.AbsTaskInstructionRetrieval import AbsTaskInstructionRetrieval
from mteb.abstasks.AbsTaskRetrieval import AbsTaskRetrieval

logging.basicConfig(level=logging.INFO)


@pytest.mark.parametrize("task", MTEB().tasks_cls)
@patch("datasets.load_dataset")
def test_load_data(mock_load_dataset: Mock, task: AbsTask):
    # TODO: We skip because this load_data is completely different.
    if isinstance(task, AbsTaskRetrieval) or isinstance(
        task, AbsTaskInstructionRetrieval
    ):
        pytest.skip()
    with patch.object(task, "dataset_transform") as mock_dataset_transform:
        task.load_data()
        mock_load_dataset.assert_called()

        # They don't yet but should they so they can be expanded more easily?
        if not task.is_crosslingual and not task.is_multilingual:
            mock_dataset_transform.assert_called_once()


async def check_dataset_on_hf(
    session: aiohttp.ClientSession, dataset: str, revision: str
) -> bool:
    url = f"https://huggingface.co/datasets/{dataset}/tree/{revision}"
    async with session.head(url) as response:
        return response.status == 200


async def check_datasets_are_available_on_hf(tasks):
    does_not_exist = []
    async with aiohttp.ClientSession() as session:
        tasks_checks = [
            check_dataset_on_hf(
                session,
                task.metadata.dataset["path"],
                task.metadata.dataset["revision"],
            )
            for task in tasks
        ]
        datasets_exists = await asyncio.gather(*tasks_checks)

    for task, ds_exists in zip(tasks, datasets_exists):
        if not ds_exists:
            does_not_exist.append(
                (task.metadata.dataset["path"], task.metadata.dataset["revision"])
            )

    if does_not_exist:
        pretty_print = "\n".join(
            [f"{ds[0]} - revision {ds[1]}" for ds in does_not_exist]
        )
        assert False, f"Datasets not available on Hugging Face:\n{pretty_print}"


def test_dataset_availability():
    """Checks if the datasets are available on Hugging Face using both their name and revision."""
    tasks = MTEB().tasks_cls
    asyncio.run(check_datasets_are_available_on_hf(tasks))
